/*
		Obt: para funciones que obtienen datos
		Asig: para funciones que asignan datos
		Impr: para funciones que imprimen datos en pantalla

*/


/*
	Guarda informacin de un archivo de audio para poder ser usada convenientemente
	'na' es llenado cuando se abre el archivo de audio usando 'mus_audio_open_imput'
	que devuelve un nmero nico de archivo o -1 si no se puede abrir el archivo.
	'muestras' y 'mu_can' representan mnbos un valor en muestras; el primero se
	refiere al nmero total de muestras que tiene el archivo mientras que el segundo
	a la cantidad de muestras por canal.
*/
typedef struct 
{
  int na;                /* numero de archivo asociado, o -1 si se falla la apertura del archivo */
  int canales;				   /* canales del audio */
  int frec_muestreo;     /* frecuencia de muestreo */
  off_t muestras;        /* muestras en total que tiene el archivo de audio */
  off_t mu_can;          /* se refiere a la cantidad de muestras en cada canal */
  float duracion;        /* la duracin en segundos del archivo de audio */
  int formato;           /* el formato que tiene el archivo de audio */
  int cabecera;
  char *comentario;      /* el comentario que pueda tener el archivo de audio, de otro modo es NULL */
} DAUDIO;

#define TAM_BUFFER 1000   /*  tamao de un espacio de memoria a usar */


/* 
	chequea que la entrada de la lnea de comandos tenga el nmero de argumentos necesariso 

	args : la cantidad de argumentos que el usuario ingres.
	nargs : la cantidad argumentos necesarios para correr sta aplicacin.
	msj : el mensaje que se va a mostrar, dependiendo de la aplicacin.
	nomb_exe : nombre del ejecutable de sta aplicacin.
*/
int ChequearEntrada(int args, int nargs, char *msj, char *nomb_exe)
{
  if(args < nargs) 
		{
    fprintf(stderr, "\n  %s: insuficientes argumentos [%d] \n  %s", nomb_exe, args, msj);
    fprintf(stdout, "\n\n Presione aceptar para terminar el programa ..."); 
    return -1;
		}
  return 0; 
}


/* 
	Obtiene los datos del audio  que se la pasa como nombre de archivo y los guarda en 'dau'
	Como se ve, la estructura se pasa en la lista de argumentos y no se devuelve porque ya 
	fu creada ntes, si no fuera as habria que crearla dentro y devolverla

	narch: nombre completo del archivo de audio, esto puede incluir la ruta.
	dau: la estructura que contendr todos los datos del encabezado del archivo de audio.

	Se usan las funciones de sndlib para leer distintos datos del encabezado de un audio.
*/
void ObtDatosDeAudio(char *narch, DAUDIO *dau)
{
	dau->frec_muestreo = mus_sound_srate(narch);
	dau->canales = mus_sound_chans(narch);
	dau->muestras = mus_sound_samples(narch);
	dau->mu_can  = mus_sound_frames(narch);                /* los mu_can NO SON lo mismo que las muestras...!*/
	dau->comentario = mus_sound_comment(narch); 
	dau->formato = mus_sound_data_format(narch);
	dau->cabecera = mus_sound_header_type(narch);
  dau->duracion = (double)dau->mu_can / (float)dau->frec_muestreo;
}


/* 
	Imprime en la consola informacin del archivo de audio guardados en 'dau'.
	Tambin se le pasa el nombre del archivo para obtener el tipo de la cabecera 
	y el formato del audio, stos datos no estn guardados en la estructura DAUDIO. 
	Por ltmo, se permite pasar un mensaje a ser mostrado junto con los datos como ttulo.

	msj: es utilizado como un ttulo que se agrega ntes del nombre del archvio de audio.
*/
void ImprDatosDeAudio(char *narch, DAUDIO *dau, char *msj)
{
	fprintf(stdout, "\n%s: %s:\n\n  F.MUESTREO: \t%d  m/s\n  CANALES: \t%d\n  DURACION: \t%f  seg\n  MUESTRAS: \t%d\n  MUES/CAN: \t%d\n",
		msj, narch, dau->frec_muestreo, dau->canales, dau->duracion, dau->muestras, dau->mu_can);

  fprintf(stdout, "  TIPO: \t%s\n  FORMATO: \t%s\n  COMENTARIO: \t%s\n", 
								  mus_header_type_name(mus_sound_header_type(narch)),
									mus_data_format_name(mus_sound_data_format(narch)),
									dau->comentario);
}


/* 
	Abre un archivo de audio para ser usado como entrada.
	La funcin realiza otras operaciones. Luego de que la apertura del archivo
	fue correcta, obtiene sus datos y los impirme en pantalla. Estas operaciones adicionales
	nos servirn como controles

	Para eso, primero se debe abrir el archivo. Si todo sale bien, entonces
	obtener sus datos y luego imprimirlos en pantalla. La impresin de los datos en 
	pantalla no es necesaria para sta funcin pero nos asegura a nosotros que la operacin
	de escrbir los datos es correcta, ya que el nico control que la funcin hace es la 
	de si se pudo abrir el archivo.

	narch: nombre completo del archivo
	msj: el mensaje que es usado como ttulo al mostrar los datos en pantalla

*/
DAUDIO *AbrirAudioEntrada(char* narch, char* msj)
{
  DAUDIO *dau;                                     /* creamos un objeto de tipo DAUDIO */
  
	dau = (DAUDIO *)malloc(sizeof(DAUDIO));          /* le creamos memoria */
	dau->na = mus_sound_open_input(narch);           /* snd abre el archivo de audio */
  if (dau->na != -1)                               /* si no fall la apertura del archivo... */
    {
		ObtDatosDeAudio(narch, dau);                    /* obtenemos sus datos y lo cargamos en la estructura */
		ImprDatosDeAudio(narch, dau, msj);               /* imprimimos en pantalla los datos */
    }
  else                                             /* ...no se pudo abrir el archivo */
		{
    fprintf(stderr, "%s: %s\n", narch, strerror(errno));
    fprintf(stdout,"\n\n Presione aceptar para terminar el programa ..."); 
    getchar();
    return NULL;
		}
  return dau;                                      /* todo bien, retornamos la estructura cargada */
}


/* 
	abre el audio para la salida y rellena una estructura con sus datos 
  se devuelve la estructura con los datos nuevos para uso posterior. si se quiere 
	
	Al igual que con la funcin anterior tambin se imprimen los datos en pantalla
*/
DAUDIO *AbrirAudioSalida(char* nombre, DAUDIO *datos, char* msj)
{
  DAUDIO *dsal;

  dsal = (DAUDIO *)malloc(sizeof(DAUDIO));
  dsal->na = mus_sound_open_output(nombre, datos->frec_muestreo, datos->canales, 
				       datos->formato, datos->cabecera, "Creado por un alumno de la EUM");
  if (dsal->na != -1)
    {
    dsal->frec_muestreo = datos->frec_muestreo;
    dsal->canales = datos->canales;
    dsal->formato = datos->formato;
    dsal->cabecera = datos->cabecera;
    dsal->mu_can = datos->mu_can;
    dsal->muestras = datos->muestras;
    dsal->comentario = "Creado por un alumno de la EUM";
    dsal->duracion = (double)dsal->muestras / (float)(dsal->canales * dsal->frec_muestreo);
		ImprDatosDeAudio(nombre, dsal, msj);
    }
  else 
		{
    fprintf(stderr, "%s: %s\n", nombre, strerror(errno));
    fprintf(stdout,"\n\n Presione aceptar para terminar el programa ..."); 
    getchar();
    return NULL;
		}
  return dsal;
}


/* Crea memoria para ser usado por el snd. Toma en cuenta el nmero de canales y el
tamao del BUFFER, creo */
mus_sample_t **SndAsignarMemoria(DAUDIO *dau, long int tam)
{
  mus_sample_t **buf;
  int i;

	/* CALLOC es una macro creada en sndlib.h para reservar memoria	*/
  buf = (mus_sample_t **)CALLOC(dau->canales, sizeof(int *));
  for(i = 0; i < dau->canales; i++) 
		buf[i] = (mus_sample_t *)CALLOC(tam, sizeof(mus_sample_t));

  return buf;
}


/* Libera la memoria creada anteriormente para el snd */
void SndLiberarMemoria(DAUDIO *dau, long int tam, mus_sample_t **buf)
{
  int i;
		/* Debido a que sta memoria fue creada con CALLOC se usa FREE para liberarla */
  for(i = 0; i < dau->canales; i++) 
		FREE(buf[i]);
  FREE(buf);
}


