

#include "sndlib/io.h"
#include "sndlib/headers.h"
#include "sndlib/sound.h"

#include "funcs/es_funcs.h"


void Procesar(DAUDIO *entr, mus_sample_t **ebuf, DAUDIO *sal, mus_sample_t **sbuf);

int main(int argc, char *argv[])
{
	DAUDIO *au_entr = NULL, *au_sal = NULL;
  mus_sample_t **ebuf, **sbuf;    /* buffer usados para la entrada y salida de las muestras */

			/* ahora chequeo que los argumentos sean los correctos */
	if(ChequearEntrada(argc, 3, "Lnea de comando: falta nombre de archivo", "sndescrenc") == -1)
		{
    getchar();
    return -1;
		}
  mus_sound_initialize();	  /* inicializo sndlib */

		/* abro el audio de entrada que automticamente imprime sus datos en pantalla,
			 tambon me fijo que el archivo se pueda abrir, de otro modo salgo con error */
	au_entr = AbrirAudioEntrada(argv[1], "\nAUDIO DE ENTRADA");
	if(au_entr == NULL)
    return -1;
		/*
		dado que el archivo de salida es una modificacin basada en el de entrada, se usan los
		datos del archivo de entrada para llenar los de salida.
		*/
	au_sal = AbrirAudioSalida(argv[2], au_entr, "\nAUDIO DE SALIDA");   
	if(au_sal == NULL)
    return -1;

	/* RESERVA DE MEMORIA para los dos bufferes */
	ebuf = SndAsignarMemoria(au_entr, TAM_BUFFER);
	sbuf = SndAsignarMemoria(au_sal, TAM_BUFFER);

	Procesar(au_entr, ebuf, au_sal, sbuf);

	/* LIBERAR MEMORIA, CERRAR ARCHIVOS */
	SndLiberarMemoria(au_entr, TAM_BUFFER, ebuf);
	SndLiberarMemoria(au_sal, TAM_BUFFER, sbuf);
  close(au_entr->na);
  close(au_sal->na);
	free(au_entr);
  free(au_sal);
  return 0;
}

/*
	procesa las muestra de un archivo de entrada guardandolo en uno de salida.
	En general, sta ser la forma de trabajar. Para trabajar sobre cada muestra se usan tres bucles
	el primero avanza cada tamao del buffer hasta la cantidad total de muestras.
	el segundo avanza de a uno por cada canal
	el tercero lo hace de a cada muestra hasta el tamao del buffer.

	Los bufferes son arreglos de dos dimensiones, en la primera dimensin representa cada canal,
	mientras que la segudna dimensin, cada muestra en dicho canal.

	'mus_sound_read' lee las muestras de un archivo ya abierto teniendo en cuenta 
	                 la cantidad de canales que tiene. Se guardan stos datos en un buffer de dos dimensiones,
									 o sea que sta funcin ordena las muestras por cada canal.
									 En la funcin 'Procesar' siempre se lee de a TAM_BUFFER,
									 aunque se tiene en cuenta de no sobrepasarse, debido a que la cantidad de muestras
									 no tiene porque ser mltiplo de TAM_BUFFER.

	 ASUMO: que en el archivo las muestras estn contiguas siempre, es decir si hay dos canales
					cada muestra de cada canal se guardan una junto a la otra, y no es que primero se 
					guardan las muestras de un canal y luego las de otro. PREGUNTAR


*/
void Procesar(DAUDIO *entr, mus_sample_t **ebuf, DAUDIO *sal, mus_sample_t **sbuf)
{
	int i, k, j;             /* iteradores */
	int tam = TAM_BUFFER;    /* se usa porque 'tam' puede ser modificado */
	float mues;             /* muestra actual modificada */

	for(i = 0; i < sal->muestras; i += TAM_BUFFER)     /* itera sobre todas las muestras, avanzando cada tamao de buffer */
		{
    mus_sound_read(entr->na, 0, tam - 1, entr->canales, ebuf);  
		for(j = 0; j < sal->canales; j++)                /* itera por cada canal */
			{
			for(k = 0; k < tam; k++)          /* itera hasta el tamao del buffer o hasta que no haya ms muestras */
				{
				if(i + k == sal->muestras)   /* preveo de no pasarme de largo en el arreglo */
					break; 
				mues = MUS_SAMPLE_TO_FLOAT(ebuf[j][k]);   /* obtengo un muestra de un canal especfico */
					/* 
						como se ve, se convierte primero las muestras a float para poder trabajr con ellas
						y luego los floats a smaples para guardarlos en el buffer de salida
						es aqu donde se hacen todas las operaciones de manipulacin de cada muestra
						en ste caso no se hace nada dado que lo que interesa es mostrar la estructura del proceso
					*/
				sbuf[j][k] = MUS_FLOAT_TO_SAMPLE(mues);  /* guardo la modificacin en la salida */
				}
			}
     if(k != TAM_BUFFER && k != 0) 
				tam = j - 1;
			mus_sound_write(sal->na, 0, tam - 1,sal->canales, sbuf);   /* escribe en archivo de salida */
		}
  mus_sound_close_input(entr->na); 
  mus_sound_close_output(sal->na, sal->muestras * mus_bytes_per_sample(sal->formato));

  //fprintf(stdout,"\n\n %ld frames of %ld samples each one written", ((i-BUFFER_SIZE)+j),out->chans );
}



